@@ -1,11 +1,14 @@
 <?php
 
-/** @generate-function-entries */
+/**
+ * @generate-function-entries
+ * @generate-legacy-arginfo 70000
+ */
 
 class Imagick
 {
-#if MagickLibVersion > 0x628
-    public function optimizeImageLayers(): bool  {}
+
+    public function optimizeImageLayers(): Imagick {}
 
     // METRIC_*
     public function compareImageLayers(int $metric): Imagick  {}
@@ -114,13 +117,13 @@ class Imagick
     /** @deprecated */
     public function transformImage(string $crop, string $geometry): Imagick  {}
 #endif
-#endif
 
 #if MagickLibVersion > 0x630
 #if MagickLibVersion < 0x700
     /** @deprecated */
     public function setImageOpacity(float $opacity): bool  {}
 #endif
+#endif
 
 #if MagickLibVersion >= 0x700
     public function setImageAlpha(float $alpha): bool {}
@@ -134,7 +137,6 @@ class Imagick
         int $channel = Imagick::CHANNEL_DEFAULT
     ): bool  {}
 #endif
-#endif
 
 #if MagickLibVersion >= 0x700
     public function polaroidWithTextAndMethod(
@@ -145,7 +147,6 @@ class Imagick
     ): bool {}
 #endif
 
-#if MagickLibVersion > 0x631
 
     public function polaroidImage(ImagickDraw $settings, float $angle): bool  {}
 
@@ -175,7 +176,7 @@ class Imagick
     public function getImageLength(): int  {}
 
     public function extentImage(int $width, int $height, int $x, int $y): bool  {}
-#endif
+
 #if MagickLibVersion > 0x633
     public function getImageOrientation(): int  {}
 
@@ -198,11 +199,15 @@ class Imagick
 #endif
 #endif
 
-#if MagickLibVersion > 0x635
-
-    // TODO - Imagick
     public function clutImage(Imagick $lookup_table, int $channel = Imagick::CHANNEL_DEFAULT): bool  {}
 
+#if MagickLibVersion >= 0x700
+    public function clutImageWithInterpolate(
+        Imagick $lookup_table,
+        int $pixel_interpolate_method // PixelInterpolateMethod
+    ): bool  {}
+#endif
+
     public function getImageProperties(string $pattern = "*", bool $include_values = true): array  {}
 
     public function getImageProfiles(string $pattern = "*", bool $include_values = true): array  {}
@@ -237,7 +242,7 @@ class Imagick
     public function recolorImage(array $matrix): bool  {}
 #endif
 #endif
-#endif
+
 
 #if MagickLibVersion > 0x636
     public function setFont(string $font): bool  {}
@@ -292,7 +297,6 @@ class Imagick
 
     public function encipherImage(string $passphrase): bool  {}
 
-//    PHP_ME(imagick, decipherimage, imagick_decipherimage_args, ZEND_ACC_PUBLIC)
     public function decipherImage(string $passphrase): bool  {}
 #endif
 
@@ -491,7 +495,7 @@ class Imagick
 
     public function getImageSize(): int  {}
 
-    public function getImageBlob(): string  {}
+    public function getImageBlob(): ?string  {}
 
     public function getImagesBlob(): string  {}
 
@@ -802,8 +806,12 @@ class Imagick
 
     public function combineImages(int $colorspace): Imagick  {}
 
+#if MagickLibVersion >= 0x700
+    public function convolveImage(ImagickKernel $kernel, int $channel = Imagick::CHANNEL_DEFAULT): bool  {}
+#else
     // kernel is a 2d array of float values
     public function convolveImage(array $kernel, int $channel = Imagick::CHANNEL_DEFAULT): bool  {}
+#endif
 
     public function cycleColormapImage(int $displace): bool  {}
 
@@ -827,7 +835,7 @@ class Imagick
 //http://www.imagemagick.org/Usage/layers/#evaluate-sequence
 
     // EVALUATE_*
-    public function evaluateImages(int $evaluate): bool {}
+    public function evaluateImages(int $evaluate): Imagick {}
 
 #endif
 
@@ -1079,7 +1087,11 @@ class Imagick
     public function setImageBiasQuantum(string $bias): void  {}
 #endif
 
+#if MagickLibVersion >= 0x700
+    public function setImageBluePrimary(float $x, float $y, float $z): bool  {}
+#else
     public function setImageBluePrimary(float $x, float $y): bool  {}
+#endif
     /* {{{ proto bool Imagick::setImageBluePrimary(float x,float y)
 For IM7 the prototype is
 proto bool Imagick::setImageBluePrimary(float x, float y, float z) */
@@ -1096,21 +1108,33 @@ proto bool Imagick::setImageBluePrimary(float x, float y, float z) */
 
     public function setImageExtent(int $columns, int $rows): bool  {}
 
+#if MagickLibVersion >= 0x700
+    public function setImageGreenPrimary(float $x, float $y, float $z): bool  {}
+#else
     public function setImageGreenPrimary(float $x, float $y): bool  {}
+#endif
 
     // INTERLACE_*
     public function setImageInterlaceScheme(int $interlace): bool  {}
 
     public function setImageProfile(string $name, string $profile): bool  {}
 
+#if MagickLibVersion >= 0x700
+    public function setImageRedPrimary(float $x, float $y, float $z): bool  {}
+#else
     public function setImageRedPrimary(float $x, float $y): bool  {}
+#endif
 
     // RENDERINGINTENT
     public function setImageRenderingIntent(int $rendering_intent): bool  {}
 
     public function setImageVirtualPixelMethod(int $method): bool  {}
 
+#if MagickLibVersion >= 0x700
+    public function setImageWhitePoint(float $x, float $y, float $z): bool  {}
+#else
     public function setImageWhitePoint(float $x, float $y): bool  {}
+#endif
 
     public function  sigmoidalContrastImage(
         bool $sharpen,
@@ -1150,7 +1174,7 @@ proto bool Imagick::setImageBluePrimary(float x, float y, float z) */
         int $columns,
         int $rows,
         ImagickPixel|string $background_color,
-        string $format = null
+        ?string $format = null
     ): bool  {}
 
     // TODO - canvas? description
@@ -1197,7 +1221,7 @@ proto bool Imagick::setImageBluePrimary(float x, float y, float z) */
 
     public static function getResource(int $type): int  {}
 
-    public static function getResourceLimit(int $type): int  {}
+    public static function getResourceLimit(int $type): float  {}
 
     public function getSamplingFactors(): array  {}
 
@@ -1246,16 +1270,10 @@ proto bool Imagick::setImageBluePrimary(float x, float y, float z) */
 //    public function key(): int  {}
 //
 //# endif
-//#endif
+#endif
 
-    /** @alias Imagick::nextImage
-     *  @tentative-return-type
-     */
     public function next(): void  {}
 
-    /** @alias Imagick::setFirstIterator
-     *  @tentative-return-type
-     */
     public function rewind(): void  {}
 
     public function valid(): bool  {}
@@ -1295,15 +1313,23 @@ proto bool Imagick::setImageBluePrimary(float x, float y, float z) */
 #endif
 
 #if MagickLibVersion >= 0x652
-    public function subimageMatch(Imagick $image, ?array &$offset = null, ?float &$similarity = null, float $threshold = 0.0, int $metric = 0): Imagick  {}
+	/**
+	 * @param array $offset
+	 * @param float $similarity
+	 */
+    public function subimageMatch(Imagick $image, &$offset = null, &$similarity = null, float $threshold = 0.0, int $metric = 0): Imagick  {}
 
-    /** @alias Imagick::subimageMatch */
-    public function similarityImage(Imagick $image, ?array &$offset = null, ?float &$similarity = null, float $threshold = 0.0, int $metric = 0): Imagick  {}
+	/**
+	 * @alias Imagick::subimageMatch
+	 * @param array $offset
+	 * @param float $similarity
+	 */
+    public function similarityImage(Imagick $image, &$offset = null, &$similarity = null, float $threshold = 0.0, int $metric = 0): Imagick  {}
 #endif
 
     public static function setRegistry(string $key, string $value): bool  {}
 
-    public static function getRegistry(string $key): string  {}
+    public static function getRegistry(string $key): string|false  {}
 
     public static function listRegistry(): array {}
 
@@ -1368,7 +1394,7 @@ proto bool Imagick::setImageBluePrimary(float x, float y, float z) */
 #endif
 
 #if MagickLibVersion >= 0x693
-    public function localContrastImage(float $radius, float $strength): void {}
+    public function localContrastImage(float $radius, float $strength): bool {}
 #endif
 
 #if MagickLibVersion >= 0x700
@@ -1402,7 +1428,7 @@ proto bool Imagick::setImageBluePrimary(float x, float y, float z) */
 //        int $channel = Imagick::CHANNEL_DEFAULT): null {}
 
 
-#if MagickLibVersion >= 0x709
+#if IM_HAVE_IMAGICK_CANNYEDGEIMAGE
     public function cannyEdgeImage(
         float $radius,
         float $sigma,
