@@ -40,8 +40,8 @@ void
 static void php_imagickkernelvalues_to_zval(zval *zv, KernelInfo *kernel_info) {
 	int count;
 	double value;
-	int x, y;
-#ifdef ZEND_ENGINE_3
+	unsigned int x, y;
+#if PHP_VERSION_ID >= 70000
 	zval row;
 #else
 	zval *row;
@@ -52,7 +52,7 @@ static void php_imagickkernelvalues_to_zval(zval *zv, KernelInfo *kernel_info) {
 	count = 0;
 
 	for (y=0; y<kernel_info->height ; y++) {
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 		p_row = &row;
 #else
 		MAKE_STD_ZVAL(row);
@@ -79,12 +79,16 @@ static void php_imagickkernelvalues_to_zval(zval *zv, KernelInfo *kernel_info) {
 }
 
 
+#if PHP_VERSION_ID >= 80000
+HashTable* php_imagickkernel_get_debug_info(zend_object *obj, int *is_temp TSRMLS_DC) /* {{{ */
+#else
 HashTable* php_imagickkernel_get_debug_info(zval *obj, int *is_temp TSRMLS_DC) /* {{{ */
+#endif
 {
 	php_imagickkernel_object *internp;
 	HashTable *debug_info;
 	KernelInfo *kernel_info;
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	zval matrix;
 #else
 	zval *matrix;
@@ -92,14 +96,18 @@ HashTable* php_imagickkernel_get_debug_info(zval *obj, int *is_temp TSRMLS_DC) /
 
 	*is_temp = 1; //var_dump will destroy the hashtable
 
+#if PHP_VERSION_ID >= 80000
+	internp = php_imagickkernel_fetch_object(obj);
+#else
 	internp = Z_IMAGICKKERNEL_P(obj);
+#endif
 	kernel_info = internp->kernel_info;
 
 	ALLOC_HASHTABLE(debug_info);
 	ZEND_INIT_SYMTABLE_EX(debug_info, 1, 0);
 
 	while (kernel_info != NULL) {
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 		array_init(&matrix);
 		php_imagickkernelvalues_to_zval(&matrix, kernel_info);
 		zend_hash_next_index_insert(debug_info, &matrix);
@@ -140,8 +148,6 @@ static void im_CalcKernelMetaData(KernelInfo *kernel) {
 			kernel->maximum = kernel->values[i];
 		}
 	}
-
-	return;
 }
 
 
@@ -151,10 +157,10 @@ static KernelInfo *imagick_createKernel(KernelValueType *values, size_t width, s
 	KernelInfo *kernel_info;
 
 #if MagickLibVersion >= 0x700
-	int i;
-	ExceptionInfo *exception_info = NULL;
+	unsigned int i;
+	ExceptionInfo *_exception_info = (ExceptionInfo *) NULL;
 	//TODO - inspect exception info
-	kernel_info=AcquireKernelInfo(NULL, exception_info);
+	kernel_info=AcquireKernelInfo(NULL, _exception_info);
 #else
 	kernel_info=AcquireKernelInfo(NULL);
 #endif
@@ -179,8 +185,11 @@ static KernelInfo *imagick_createKernel(KernelValueType *values, size_t width, s
 	for (i=0; i<width*height;i++) {
 		kernel_info->values[i] = (MagickRealType)values[i];
 	}
+	// The values are not going to be used so free them
+	RelinquishAlignedMemory(values);
 	
 #else
+	// For IM 6, the values get used
 	kernel_info->values = values;
 #endif
 
@@ -198,6 +207,22 @@ static void createKernelZval(zval *pzval, KernelInfo *kernel_info TSRMLS_DC) {
 	intern_return->kernel_info = kernel_info;
 }
 
+
+/* {{{ proto ImagickKernel ImagickKernel::__construct()
+   The ImagickKernel constructor
+*/
+PHP_METHOD(ImagickKernel, __construct)
+{
+	// This suppresses an 'unused parameter' warning.
+	(void)return_value;
+
+	if (zend_parse_parameters_none() == FAILURE) {
+		RETURN_THROWS();
+	}
+	// this method is private.
+}
+/* }}} */
+
 #define MATRIX_ERROR_EMPTY "Cannot create kernel, matrix is empty."
 #define MATRIX_ERROR_UNEVEN "Values must be matrix, with the same number of columns in each row."
 #define MATRIX_ERROR_BAD_VALUE "Only numbers or false are valid values in a kernel matrix."
@@ -209,16 +234,16 @@ static void createKernelZval(zval *pzval, KernelInfo *kernel_info TSRMLS_DC) {
 	matrixes that are odd sizes in both dimensions the the origin pixel will default 
 	to the centre of the kernel. For all other kernel sizes the origin pixel must be specified.
 */
-#ifdef ZEND_ENGINE_3
-PHP_METHOD(imagickkernel, frommatrix)
+#if PHP_VERSION_ID >= 70000
+PHP_METHOD(ImagickKernel, fromMatrix)
 {
 	zval *kernel_array;
 	zval *origin_array;
 	HashTable *inner_array;
 	KernelInfo *kernel_info;
-	long num_rows, num_columns;
-	int previous_num_columns;
-	int row, column;
+	unsigned long num_rows, num_columns = 0;
+	unsigned int previous_num_columns = (unsigned int)-1;
+	unsigned int row, column;
 
 	zval *pzval_outer;
 	zval *pzval_inner;
@@ -230,13 +255,12 @@ PHP_METHOD(imagickkernel, frommatrix)
 	KernelValueType *values = NULL;
 	double notanumber = sqrt((double)-1.0);  /* Special Value : Not A Number */
 
-	previous_num_columns = -1;
 	count = 0;
 	row = 0;
 	origin_array = NULL;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a|a", &kernel_array, &origin_array) == FAILURE) {
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a|a!", &kernel_array, &origin_array) == FAILURE) {
+		RETURN_THROWS();
 	}
 
 	num_rows = zend_hash_num_elements(Z_ARRVAL_P(kernel_array));
@@ -244,7 +268,7 @@ PHP_METHOD(imagickkernel, frommatrix)
 	if (num_rows == 0) {
 		//error - array has zero elements.
 		php_imagick_throw_exception(IMAGICKKERNEL_CLASS, MATRIX_ERROR_EMPTY TSRMLS_CC);
-		return;
+		RETURN_THROWS();
 	}
 
 
@@ -271,7 +295,7 @@ PHP_METHOD(imagickkernel, frommatrix)
 				values = (KernelValueType *)AcquireAlignedMemory(num_columns, num_rows*sizeof(KernelValueType));
 			}
 
-			if (previous_num_columns != -1) {
+			if (previous_num_columns != ((unsigned int)-1)) {
 				if (previous_num_columns != num_columns) {
 					php_imagick_throw_exception(IMAGICKKERNEL_CLASS, MATRIX_ERROR_UNEVEN TSRMLS_CC);
 					goto cleanup;
@@ -324,6 +348,8 @@ PHP_METHOD(imagickkernel, frommatrix)
 	else {
 		HashTable *origin_array_ht;
 		origin_array_ht = Z_ARRVAL_P(origin_array);
+
+		// parse the origin_x
 		tmp = zend_hash_index_find(origin_array_ht, 0);
 		if (tmp != NULL) {
 			ZVAL_DEREF(tmp);
@@ -333,6 +359,19 @@ PHP_METHOD(imagickkernel, frommatrix)
 			php_imagick_throw_exception(IMAGICKKERNEL_CLASS, MATRIX_ORIGIN_REQUIRED TSRMLS_CC);
 			goto cleanup;
 		}
+		// origin_x is unsigned, so checking for > num_columns, also
+		// checks for < 0
+		if (origin_x>=num_columns) {
+			zend_throw_exception_ex(
+				php_imagickkernel_exception_class_entry,
+				5 TSRMLS_CC,
+				"origin_x for matrix is outside bounds of columns: %zu",
+				origin_x
+			);
+			goto cleanup;
+		}
+
+		// parse the origin_y
 		tmp = zend_hash_index_find(origin_array_ht, 1);
 		if (tmp != NULL) {
 			ZVAL_DEREF(tmp);
@@ -342,11 +381,23 @@ PHP_METHOD(imagickkernel, frommatrix)
 			php_imagick_throw_exception(IMAGICKKERNEL_CLASS, MATRIX_ORIGIN_REQUIRED TSRMLS_CC);
 			goto cleanup;
 		}
+		// origin_y is unsigned, so checking for > num_rows, also
+		// checks for < 0
+		if (origin_y>=num_rows) {
+			zend_throw_exception_ex(
+				php_imagickkernel_exception_class_entry,
+				5 TSRMLS_CC,
+				"origin_y for matrix is outside bounds of rows: %zu",
+				origin_x
+			);
+			goto cleanup;
+		}
 	}
 
 	kernel_info = imagick_createKernel(values, num_columns, num_rows, origin_x, origin_y);
 	createKernelZval(return_value, kernel_info TSRMLS_CC);
 
+	// values are freed / used by imagick_createKernel so do not try to free them
 	return;
 
 cleanup:
@@ -355,18 +406,18 @@ cleanup:
 	}
 }
 
-#else //not ZEND_ENGINE_3
+#else // PHP 5
 
 
-PHP_METHOD(imagickkernel, frommatrix)
+PHP_METHOD(ImagickKernel, fromMatrix)
 {
 	zval *kernel_array;
 	zval *origin_array;
 	HashTable *inner_array;
 	KernelInfo *kernel_info;
-	long num_rows, num_columns;
-	int previous_num_columns;
-	int row, column;
+	unsigned long num_rows, num_columns = 0;
+	unsigned int previous_num_columns = (unsigned int)-1;
+	unsigned int row, column;
 
 	HashTable *origin_array_ht;
 	zval **ppzval_outer;
@@ -384,8 +435,8 @@ PHP_METHOD(imagickkernel, frommatrix)
 	row = 0;
 	origin_array = NULL;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a|a", &kernel_array, &origin_array) == FAILURE) {
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a|a!", &kernel_array, &origin_array) == FAILURE) {
+		RETURN_THROWS();
 	}
 
 	num_rows = zend_hash_num_elements(Z_ARRVAL_P(kernel_array));
@@ -393,7 +444,7 @@ PHP_METHOD(imagickkernel, frommatrix)
 	if (num_rows == 0) {
 		//error - array has zero elements.
 		php_imagick_throw_exception(IMAGICKKERNEL_CLASS, MATRIX_ERROR_EMPTY TSRMLS_CC);
-		return;
+		RETURN_THROWS();
 	}
 
 	for (row=0 ; row<num_rows ; row++) {
@@ -418,7 +469,7 @@ PHP_METHOD(imagickkernel, frommatrix)
 				values = (KernelValueType *)AcquireAlignedMemory(num_columns, num_rows*sizeof(KernelValueType));
 			}
 
-			if (previous_num_columns != -1) {
+			if (previous_num_columns != ((unsigned int)-1)) {
 				if (previous_num_columns != num_columns) {
 					php_imagick_throw_exception(IMAGICKKERNEL_CLASS, MATRIX_ERROR_UNEVEN TSRMLS_CC);
 					goto cleanup;
@@ -468,6 +519,8 @@ PHP_METHOD(imagickkernel, frommatrix)
 	}
 	else {
 		origin_array_ht = Z_ARRVAL_P(origin_array);
+
+		// parse and check the origin_x
 		if (zend_hash_index_find(origin_array_ht, 0, (void**)&tmp) == SUCCESS) {
 			origin_x = Z_LVAL_PP(tmp);
 		}
@@ -476,6 +529,19 @@ PHP_METHOD(imagickkernel, frommatrix)
 			goto cleanup;
 		}
 
+		// origin_x is unsigned, so checking for > num_columns, also
+		// checks for < 0
+		if (origin_x>=num_columns) {
+			zend_throw_exception_ex(
+				php_imagickkernel_exception_class_entry,
+				5 TSRMLS_CC,
+				"origin_x for matrix is outside bounds of columns: %d",
+				origin_x
+			);
+			goto cleanup;
+		}
+
+        // parse and check the origin_y
 		if (zend_hash_index_find(origin_array_ht, 1, (void**)&tmp) == SUCCESS) {
 			origin_y = Z_LVAL_PP(tmp);
 		}
@@ -483,6 +549,18 @@ PHP_METHOD(imagickkernel, frommatrix)
 			php_imagick_throw_exception(IMAGICKKERNEL_CLASS, MATRIX_ORIGIN_REQUIRED TSRMLS_CC);
 			goto cleanup;
 		}
+
+		// origin_y is unsigned, so checking for > num_rows, also
+		// checks for < 0
+		if (origin_y>=num_rows) {
+			zend_throw_exception_ex(
+				php_imagickkernel_exception_class_entry,
+				5 TSRMLS_CC,
+				"origin_y for matrix is outside bounds of rows: %d",
+				origin_y
+			);
+			goto cleanup;
+		}
 	}
 
 	kernel_info = imagick_createKernel(values, num_columns, num_rows, origin_x, origin_y);
@@ -569,33 +647,38 @@ static void imagick_fiddle_with_geometry_info(ssize_t type, GeometryFlags flags,
  Create a kernel from a builtin in kernel. See http://www.imagemagick.org/Usage/morphology/#kernel for examples. Currently the 'rotation' symbols are not supported. Example:
  $diamondKernel = ImagickKernel::fromBuiltIn(\Imagick::KERNEL_DIAMOND, "2");
 */
-PHP_METHOD(imagickkernel, frombuiltin)
+PHP_METHOD(ImagickKernel, fromBuiltin)
 {
 	im_long kernel_type;
-	GeometryInfo geometry_info = {0};
+	GeometryInfo geometry_info = {
+		0, //rho,
+		0, //sigma,
+		0, //xi,
+		0, //psi,
+		0, //chi;
+	};
 	KernelInfo *kernel_info;
 	char *string;
 	IM_LEN_TYPE string_len;
 	GeometryFlags flags;
+#if MagickLibVersion >= 0x700
+	ExceptionInfo *_exception_info = NULL;
+#endif
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ls", &kernel_type, &string, &string_len) == FAILURE) {
-		return;
+		RETURN_THROWS();
 	}
 
 	flags = ParseGeometry(string, &geometry_info);
 	imagick_fiddle_with_geometry_info(kernel_type, flags, &geometry_info);
 	
 #if MagickLibVersion >= 0x700
-
-	ExceptionInfo *exception_info = NULL;
 	//TODO - inspect exception info
-	kernel_info = AcquireKernelBuiltIn(kernel_type, &geometry_info, exception_info);
+	kernel_info = AcquireKernelBuiltIn(kernel_type, &geometry_info, _exception_info);
 #else
 	kernel_info = AcquireKernelBuiltIn(kernel_type, &geometry_info);
 #endif
 	createKernelZval(return_value, kernel_info TSRMLS_CC);
-
-	return;
 }
 
 /* }}} */
@@ -604,10 +687,9 @@ PHP_METHOD(imagickkernel, frombuiltin)
 /* {{{ proto void ImagickKernel::addKernel(ImagickKernel kernel)
 	Attach another kernel to this kernel to allow them to both be applied in a single morphology or filter function. Returns the new combined kernel.
 */
-PHP_METHOD(imagickkernel, addkernel)
+PHP_METHOD(ImagickKernel, addKernel)
 {
 	zval *objvar;
-//	KernelInfo *kernel_info_this_clone;
 	KernelInfo *kernel_info_add_clone;
 
 	KernelInfo *kernel_info;
@@ -617,24 +699,25 @@ PHP_METHOD(imagickkernel, addkernel)
 	php_imagickkernel_object *internp;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "O", &objvar, php_imagickkernel_sc_entry) == FAILURE) {
-		return;
+		RETURN_THROWS();
 	}
 
 	kernel = Z_IMAGICKKERNEL_P(objvar);
 	internp = Z_IMAGICKKERNEL_P(getThis());
+
+	if (kernel->kernel_info == NULL) {
+		zend_throw_exception(php_imagickkernel_exception_class_entry, "ImagickKernel is empty, cannot be used", (long)0 TSRMLS_CC);
+		RETURN_NULL();
+	}
+
 	kernel_info = internp->kernel_info;
-
-	//createKernelZval(return_value, kernel_info_this_clone TSRMLS_CC);
-
-	while (kernel_info != NULL) {
+	do {
 		kernel_info_target = kernel_info;
 		kernel_info = kernel_info->next;
-	};
+	} while (kernel_info != NULL);
 
 	kernel_info_add_clone = CloneKernelInfo(kernel->kernel_info);
 	kernel_info_target->next = kernel_info_add_clone;
-
-	return;
 }
 /* }}} */
 
@@ -642,7 +725,7 @@ PHP_METHOD(imagickkernel, addkernel)
 /* {{{ proto ImagickKernel[] ImagickKernel::separate(void)
 	Separates a linked set of kernels and returns an array of ImagickKernels.
 */
-PHP_METHOD(imagickkernel, separate)
+PHP_METHOD(ImagickKernel, separate)
 {
 	php_imagickkernel_object *internp;
 	KernelInfo *kernel_info;
@@ -650,23 +733,25 @@ PHP_METHOD(imagickkernel, separate)
 	int number_values;
 	KernelValueType * values_copy;
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 	zval separate_object;
 #else
 	zval *separate_object;
 #endif
 
 	if (zend_parse_parameters_none() == FAILURE) {
-		return;
+		RETURN_THROWS();
 	}
 
 	internp = Z_IMAGICKKERNEL_P(getThis());
+	IMAGICK_KERNEL_NOT_NULL_EMPTY(internp);
 	kernel_info = internp->kernel_info;
 
 	array_init(return_value);
 
 	while (kernel_info != NULL) {
 		number_values = kernel_info->width * kernel_info->height;
+		// The values are created because they can by used by imagick_createKernel which takes their ownership
 		values_copy = (KernelValueType *)AcquireAlignedMemory(kernel_info->width, kernel_info->height*sizeof(KernelValueType));
 		memcpy(values_copy, kernel_info->values, number_values * sizeof(KernelValueType));
 
@@ -678,7 +763,7 @@ PHP_METHOD(imagickkernel, separate)
 			kernel_info->y
 		);
 
-#ifdef ZEND_ENGINE_3
+#if PHP_VERSION_ID >= 70000
 		createKernelZval(&separate_object, kernel_info_copy TSRMLS_CC);
 		add_next_index_zval(return_value, &separate_object);
 #else 
@@ -688,8 +773,6 @@ PHP_METHOD(imagickkernel, separate)
 #endif
 		kernel_info = kernel_info->next;
 	}
-
-	return;
 }
 /* }}} */
 
@@ -698,20 +781,19 @@ PHP_METHOD(imagickkernel, separate)
 	Get the 2d matrix of values used in this kernel. The elements are either 
 	float for elements that are used or 'false' if the element should be skipped.
 */
-PHP_METHOD(imagickkernel, getmatrix)
+PHP_METHOD(ImagickKernel, getMatrix)
 {
 	php_imagickkernel_object *internp;
 
 	if (zend_parse_parameters_none() == FAILURE) {
-		return;
+		RETURN_THROWS();
 	}
 
 	internp = Z_IMAGICKKERNEL_P(getThis());
+	IMAGICK_KERNEL_NOT_NULL_EMPTY(internp);
 
 	array_init(return_value);
 	php_imagickkernelvalues_to_zval(return_value, internp->kernel_info);
-
-	return;
 }
 /* }}} */
 
@@ -729,20 +811,20 @@ PHP_METHOD(imagickkernel, getmatrix)
 	Flag should be one of NORMALIZE_KERNEL_VALUE, NORMALIZE_KERNEL_CORRELATE, 
 	NORMALIZE_KERNEL_PERCENT or not set.
 */
-PHP_METHOD(imagickkernel, scale)
+PHP_METHOD(ImagickKernel, scale)
 {
 	php_imagickkernel_object *internp;
 	double scale;
-	im_long normalize_flag;
+	im_long normalize_flag = 0;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "d|l", &scale, &normalize_flag) == FAILURE) {
-		return;
+		RETURN_THROWS();
 	}
+	(void)return_value;
 
 	internp = Z_IMAGICKKERNEL_P(getThis());
+	IMAGICK_KERNEL_NOT_NULL_EMPTY(internp);
 	ScaleKernelInfo(internp->kernel_info, scale, normalize_flag);
-
-	return;
 }
 /* }}} */
 
@@ -755,20 +837,20 @@ PHP_METHOD(imagickkernel, scale)
 	into the resulting convolution kernel. The resulting effect is to convert the
 	defined kernels into blended soft-blurs, unsharp kernels or into sharpening kernels.
 */
-PHP_METHOD(imagickkernel, addunitykernel)
+PHP_METHOD(ImagickKernel, addUnityKernel)
 {
 	php_imagickkernel_object *internp;
 	double scale;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "d", &scale) == FAILURE) {
-		return;
+		RETURN_THROWS();
 	}
+	(void)return_value;
 
 	internp = Z_IMAGICKKERNEL_P(getThis());
+	IMAGICK_KERNEL_NOT_NULL_EMPTY(internp);
 	UnityAddKernelInfo(internp->kernel_info, scale);
-
-	return;
 }
 /* }}} */
 
-#endif
\ No newline at end of file
+#endif
